DISPLAY_TERMINAL_COLOR_TITLE_TEXT = "#000000"
DISPLAY_TERMINAL_COLOR_TITLE_BACKGROUND = "#F6DA75"
DISPLAY_TERMINAL_COLOR_SUCCESS_TEXT = "#000000"
DISPLAY_TERMINAL_COLOR_SUCCESS_BACKGROUND = "#B0D78F"
DISPLAY_TERMINAL_COLOR_ERROR_TEXT = "#000000"
DISPLAY_TERMINAL_COLOR_ERROR_BACKGROUND = "#FFAFB0"
DISPLAY_TERMINAL_COLOR_WARNING_TEXT = "#000000"
DISPLAY_TERMINAL_COLOR_WARNING_BACKGROUND = "#F2CFA5"

DISPLAY_TERMINAL_COLOR_TITLE = "cyan on purple3"
DISPLAY_TERMINAL_COLOR_SUCCESS = "black on green"
DISPLAY_TERMINAL_COLOR_ERROR = "black on red"
DISPLAY_TERMINAL_COLOR_WARNING = "black on bright_red"

DEFAULT_TASK_COLOR = "grey"

BORDER_COLOR = ""
DEFAULT_BACKGROUND_COLOR = "#171921"
DEFAULT_FONT_COLOR = "#8f929b"
TASK_COLOR = ""
CALENDAR_DATE_COLOR = "red"
CATEGORY_COLOR = ""

# Calendar date
CALENDAR_HEADER_DATE_COLOR = "#F6E2B2"
CALENDAR_WEEKDAY_NAME_COLOR = "#E6E0DB"
CALENDAR_TODAY_COLOR = "#FADD70"


# Pop up box
TABLE_TASK_NAME_COLOR = "#FAC7C1"
TABLE_TEXT_COLOR = "#D7C8B7"
TABLE_HEADER_COLOR = "#D7E1C9"
TABLE_HEADER_DATE_COLOR = "#E8E5DA"
TABLE_DEADLINE_COLOR = "#A8CCDC"
TABLE_TASK_HIGHLIGHT_COLOR = "#B1EEC9"
TABLE_TASK_DELETED_COLOR = "#808080"

# EMOJI
LEFT_ARROW_EMOJI = "⬅"
CIRCLE_EMOJI = "●"


CIRCLE_COLOR = {
    "yellow": "#f9e316",
    "green": "#29f429",
    "orange": "#EF5D0C",
    "white": "#E4DCCE",
    "brown": "#8F4802",
    "blue": "#6893F8",
    "grey": "#808080"
}

